#!/bin/env python

import sys
import gzip
import pickle
import numpy as np
from numpy import log

def npmi (beta):
    V,K = beta.shape
    p = np.mean (beta,1)
    return - (log(beta).T - log(p)) / (log(beta).T - log(K))

def view_npmi (model, k, n):
    pmi  = npmi (model['beta'])
    dic  = model['lexicon']
    stop = model['stopwords'] if 'stopwords' in model else None
    if k == 0:
        print ('topic [background]')
        p = model['background']
    else:
        print ('topic [%d]' % k)
        p = pmi[k]
    index = range (len(p))
    # prepare
    rdic = {}
    for w in dic:
        rdic[dic[w]] = w
    # body
    m = 0
    for i,q in sorted (zip(index,p), key=lambda x: x[1], reverse=True):
        if stop is None:
            print ('%s\t%.4f' % (rdic[i], q))
        else:
            print ('%s%s\t%.4f' % (rdic[i], '*' if stop[i] > 0 else '', q))
        m += 1
        if m > n:
            break

def view (model, k, n):
    beta = model['beta'].T
    dic  = model['lexicon']
    stop = model['stopwords'] if 'stopwords' in model else None
    if k == 0:
        p = model['background']
        print ('topic [background]')
    else:
        p = beta[k]
        print ('topic [%d]' % k)
    index = range (len(p))
    # prepare
    rdic = {}
    for w in dic:
        rdic[dic[w]] = w
    # body
    m = 0
    for i,q in sorted (zip(index,p), key=lambda x: x[1], reverse=True):
        if stop is None:
            print ('%s\t%.4f' % (rdic[i], q))
        else:
            print ('%s%s\t%.4f' % (rdic[i], '*' if stop[i] > 0 else '', q))
        m += 1
        if m > n:
            break
    
def pload (f):
    with gzip.open (f, 'rb') as gf:
        data = pickle.load (gf)
    return data

def rnormalize (M): # row-wise normalize matrix
    return np.array ([m / np.sum(m) for m in M])

def usage ():
    print ('usage: % viewtopic.py model [k] [n]')
    print ('$Id: viewtopic.py,v 1.2 2020/08/03 02:59:52 daichi Exp $')
    sys.exit (0)

def main ():
    if len(sys.argv) < 2:
        usage ()
    model = pload (sys.argv[1])

    if len(sys.argv) > 2:
        k = int (sys.argv[2])
        n = int (sys.argv[3]) if len(sys.argv) > 3 else 15
        view (model, k, n)
    else:
        K = model['beta'].shape[1]
        for k in range(K):
            view_npmi (model, k, 20)
            print ('------------------')

if __name__ == "__main__":
    main ()
